@@ -5,11 +5,11 @@ extern crate proc_macro;
 mod function_tests;
 
 use convert_case::{Case::Camel, Casing};
+use fancy_regex::Regex;
 use proc_macro::TokenStream;
 use proc_macro2::{Ident, Span};
 use proc_macro_error::{abort, abort_call_site, proc_macro_error};
 use quote::quote;
-use regex::Regex;
 use serde_derive::Deserialize;
 use tiny_keccak::{Hasher, Keccak};
 
@@ -26,6 +26,65 @@ struct AbiIO {
     r#type: String,
 }
 
+/// Different Contract mode will treat input and output in different
+#[derive(Debug, PartialEq)]
+enum ContractMode {
+    /// Default mode, only return the error message, if any
+    /// This is for a scenario that you just want to modify the data on
+    /// chain only
+    DefaultMode,
+    /// Rusty mode, return serialized binary of `Result<T,E>`, this is good for rust client
+    /// This is for a scenario that you are using a rust client to operation the contract
+    RustyMode,
+    /// Auto mode, return serialize binary for `T` if the Result is Ok, else return the serialize
+    /// binary of E
+    /// This is for a scenario that you take care the result but not using Rust client
+    AutoMode,
+}
+
+/// Options can set in ewasm_main function
+#[allow(dead_code)]
+#[derive(Debug, PartialEq)]
+enum ContractOption {
+    /// The default message that can be used in Default mode and Auto mode
+    DefaultMessage(String),
+}
+
+fn parse_contract_mode_and_options(
+    attr: String,
+) -> Result<(ContractMode, Vec<ContractOption>), &'static str> {
+    let (arg_str, option_str) = match attr.split_once(',') {
+        Some((head, tail)) => (head, tail),
+        None => {
+            if attr.starts_with("default") {
+                ("", attr.as_str())
+            } else {
+                (attr.as_str(), "")
+            }
+        }
+    };
+    let options = if let Ok(Some(cap)) =
+        unsafe { Regex::new(r#"default.*=.*"(?<default>[^"]*)""#).unwrap_unchecked() }
+            .captures(option_str)
+    {
+        let default = unsafe { cap.name("default").unwrap_unchecked() }.as_str();
+        vec![ContractOption::DefaultMessage(default.into())]
+    } else {
+        vec![]
+    };
+    let output = match arg_str {
+        "auto" => (ContractMode::AutoMode, options),
+        "rusty" => {
+            if options.len() != 0 {
+                return Err("can not set default message for rusty mode");
+            }
+            (ContractMode::RustyMode, options)
+        }
+        _ => (ContractMode::DefaultMode, options),
+    };
+    Ok(output)
+}
+
 fn get_function_signature(function_prototype: &str) -> [u8; 4] {
     let mut sig = [0; 4];
     let mut hasher = Keccak::v256();
@@ -36,7 +95,7 @@ fn get_function_signature(function_prototype: &str) -> [u8; 4] {
 
 fn write_function_signature(sig_str: &str) -> String {
     let re = unsafe { Regex::new(r"^(?P<name>[^(]+?)\((?P<params>[^)]*?)\)").unwrap_unchecked() };
-    if let Some(cap) = re.captures(sig_str) {
+    if let Ok(Some(cap)) = re.captures(sig_str) {
         let fn_name = unsafe { cap.name("name").unwrap_unchecked() }.as_str();
         let params = unsafe { cap.name("params").unwrap_unchecked() }
             .as_str()
@@ -69,26 +128,38 @@ fn write_function_signature(sig_str: &str) -> String {
 
 /// helps you setup the main function of a contract
 ///
-/// There are three different kind contract output.
+/// There are three different kind contract output, and the return `Result` can based on
+/// `anyhow::Result` or `std::result::Result`.  If you want to use `std::result::Result`, you need
+/// to set up `default` option with default message in default mode and auto mode, such that there
+/// will be a return message in bytes let you know some error happen.
 ///
 /// `#[ewasm_main]`
-/// The default contract output, the error will be return as a string message
+/// The default contract output, the Anyhow error will be return as a string message
 /// This is for a scenario that you just want to modify the data on
 /// chain only, and the error will to string than return.
 ///
+/// `#[ewasm_main(default="message")]`
+/// The default contract output, if any error happened the default message will be returned
+///
 /// `#[ewasm_main(rusty)]`
-/// The rust styl output, the result object from ewasm_main function will be
+/// The rust style output, the result object from ewasm_main function will be
 /// returned, this is for a scenario that you are using a rust client to catch
 /// and want to catch the result from the contract.
 ///
 /// `#[ewasm_main(auto)]`
-/// Auto unwrap the output of the result object from ewasm_main function.
+/// Auto unwrap the OK output of the Anyhow::Result object from ewasm_main function.
 /// This is for a scenario that you are using a rust non-rust client,
 /// and you are only care the happy case of executing the contract.
 ///
+/// `#[ewasm_main(auto, default="message")]`
+/// Auto unwrap the OK output of the result object (it can be `Anyhow::Result`
+/// or `std::result::Result`) from ewasm_main function, if any error happened the default message
+/// will be returned.  This is for a scenario that you are using a rust non-rust client,
+/// and you are only care the happy case of executing the contract.
+///
 /// ```compile_fail
 /// #[ewasm_main]
-/// fn main() -> Result<()> {
+/// fn main() -> anyhow::Result<()> {
 ///     let contract = Contract::new()?;
 ///     match contract.get_function_selector()? {
 ///         ewasm_fn_sig!(check_input_object) => ewasm_input_from!(contract move check_input_object)?,
@@ -97,6 +168,20 @@ fn write_function_signature(sig_str: &str) -> String {
 ///     Ok(())
 /// }
 /// ```
+/// ```compile_fail
+/// #[ewasm_main(auto, default = "Some error happen")]
+/// fn main() -> Result<String, ()> {
+///     let contract = sewup::primitives::Contract::new().expect("contract should work");
+///     match contract
+///         .get_function_selector()
+///         .expect("function selector should work")
+///     {
+///         sewup_derive::ewasm_fn_sig!(hello) => hello(),
+///         _ => Err(()),
+///     }
+/// }
+///
+/// ```
 #[proc_macro_error]
 #[proc_macro_attribute]
 pub fn ewasm_main(attr: TokenStream, item: TokenStream) -> TokenStream {
@@ -148,8 +233,41 @@ pub fn ewasm_main(attr: TokenStream, item: TokenStream) -> TokenStream {
         _ => None,
     };
 
-    match attr.to_string().to_lowercase().as_str() {
-        "auto" if Some("EwasmAny".to_string()) == output_type  => quote! {
+    let (contract_mode, options) = match parse_contract_mode_and_options(attr.to_string()) {
+        Ok(o) => o,
+        Err(e) => abort_call_site!(e),
+    };
+
+    let mut default_message = None;
+    for option in options.into_iter() {
+        let ContractOption::DefaultMessage(s) = option;
+        default_message = Some(s);
+    }
+
+    match contract_mode {
+        ContractMode::AutoMode if Some("EwasmAny".to_string()) == output_type && default_message.is_some() => quote! {
+            #[cfg(target_arch = "wasm32")]
+            use sewup::bincode;
+            #[cfg(target_arch = "wasm32")]
+            use sewup::ewasm_api::finish_data;
+            #[cfg(all(not(target_arch = "wasm32"), not(test)))]
+            pub fn main() {}
+            #[cfg(target_arch = "wasm32")]
+            #[cfg(not(any(feature = "constructor", feature = "constructor-test")))]
+            #[no_mangle]
+            pub fn main() {
+                #input
+                match #name() {
+                    Ok(r) =>  {
+                        finish_data(&r.bin);
+                    },
+                    Err(e) => {
+                        finish_data(&#default_message.as_bytes());
+                    }
+                }
+            }
+        },
+        ContractMode::AutoMode if Some("EwasmAny".to_string()) == output_type  => quote! {
             #[cfg(target_arch = "wasm32")]
             use sewup::bincode;
             #[cfg(target_arch = "wasm32")]
@@ -172,9 +290,30 @@ pub fn ewasm_main(attr: TokenStream, item: TokenStream) -> TokenStream {
                 }
             }
         },
-        // Return the inner structure from unwrap result
-        // This is for a scenario that you take care the result but not using Rust client
-        "auto" => quote! {
+        ContractMode::AutoMode if default_message.is_some() => quote! {
+            #[cfg(target_arch = "wasm32")]
+            use sewup::bincode;
+            #[cfg(target_arch = "wasm32")]
+            use sewup::ewasm_api::finish_data;
+            #[cfg(all(not(target_arch = "wasm32"), not(test)))]
+            pub fn main() {}
+            #[cfg(target_arch = "wasm32")]
+            #[cfg(not(any(feature = "constructor", feature = "constructor-test")))]
+            #[no_mangle]
+            pub fn main() {
+                #input
+                match #name() {
+                    Ok(r) =>  {
+                        let bin = bincode::serialize(&r).expect("The resuslt of `ewasm_main` should be serializable");
+                        finish_data(&bin);
+                    },
+                    Err(_) => {
+                        finish_data(&#default_message.as_bytes());
+                    }
+                }
+            }
+        },
+        ContractMode::AutoMode => quote! {
             #[cfg(target_arch = "wasm32")]
             use sewup::bincode;
             #[cfg(target_arch = "wasm32")]
@@ -198,7 +337,7 @@ pub fn ewasm_main(attr: TokenStream, item: TokenStream) -> TokenStream {
                 }
             }
         },
-        "rusty" if Some("EwasmAny".to_string()) == output_type  => quote! {
+        ContractMode::RustyMode if Some("EwasmAny".to_string()) == output_type  => quote! {
             #[cfg(target_arch = "wasm32")]
             use sewup::bincode;
             #[cfg(target_arch = "wasm32")]
@@ -215,10 +354,7 @@ pub fn ewasm_main(attr: TokenStream, item: TokenStream) -> TokenStream {
                 finish_data(&bin);
             }
         },
-
-        // Return all result structure
-        // This is for a scenario that you are using a rust client to operation the contract
-        "rusty" => quote! {
+        ContractMode::RustyMode => quote! {
             #[cfg(target_arch = "wasm32")]
             use sewup::bincode;
             #[cfg(target_arch = "wasm32")]
@@ -235,11 +371,24 @@ pub fn ewasm_main(attr: TokenStream, item: TokenStream) -> TokenStream {
                 finish_data(&bin);
             }
         },
-
-        // Default only return error message,
-        // This is for a scenario that you just want to modify the data on
-        // chain only
-        _ => quote! {
+        ContractMode::DefaultMode if default_message.is_some() => quote! {
+            #[cfg(target_arch = "wasm32")]
+            use sewup::bincode;
+            #[cfg(target_arch = "wasm32")]
+            use sewup::ewasm_api::finish_data;
+            #[cfg(all(not(target_arch = "wasm32"), not(test)))]
+            pub fn main() {}
+            #[cfg(target_arch = "wasm32")]
+            #[cfg(not(any(feature = "constructor", feature = "constructor-test")))]
+            #[no_mangle]
+            pub fn main() {
+                #input
+                if let Err(e) = #name() {
+                    finish_data(#default_message.as_bytes());
+                }
+            }
+        },
+        ContractMode::DefaultMode => quote! {
             #[cfg(target_arch = "wasm32")]
             use sewup::bincode;
             #[cfg(target_arch = "wasm32")]
@@ -260,16 +409,32 @@ pub fn ewasm_main(attr: TokenStream, item: TokenStream) -> TokenStream {
     }.into()
 }
 
-fn parse_fn_attr(fn_name: String, attr: String) -> Result<(Option<String>, String), &'static str> {
+fn parse_fn_attr(
+    fn_name: String,
+    attr: String,
+) -> Result<(Option<String>, String, Option<String>), &'static str> {
     let attr_str = attr.replace(" ", "").replace("\n", "");
     return if attr_str.is_empty() {
-        Ok((None, "{}".into()))
+        Ok((None, "{}".into(), None))
     } else if let Some((head, tail)) = attr_str.split_once(',') {
         if tail.is_empty() {
-            Ok((Some(head.replace("\"", "")), "{}".into()))
+            Ok((Some(head.replace("\"", "")), "{}".into(), None))
         } else {
+            let root_str = if let Ok(Some(cap)) =
+                unsafe { Regex::new(r"only_by=(?P<account>[^,]*)").unwrap_unchecked() }
+                    .captures(&attr_str)
+            {
+                Some(
+                    unsafe { cap.name("account").unwrap_unchecked() }
+                        .as_str()
+                        .into(),
+                )
+            } else {
+                None
+            };
+
             let mut json = "{".to_string();
-            if let Some(cap) =
+            if let Ok(Some(cap)) =
                 unsafe { Regex::new(r"constant=(?P<constant>[^,]*)").unwrap_unchecked() }
                     .captures(&attr_str)
             {
@@ -282,15 +447,17 @@ fn parse_fn_attr(fn_name: String, attr: String) -> Result<(Option<String>, Strin
                 json.push_str(r#""constant":false,"#)
             }
 
-            if let Some(cap) =
-                unsafe { Regex::new(r"inputs=(?P<inputs>\[[^\[\]]*\])").unwrap_unchecked() }
+            if let Ok(Some(cap)) =
+                unsafe { Regex::new(r#"inputs=(?<inputs>\[.*?\])(?!")"#).unwrap_unchecked() }
                     .captures(&attr_str)
             {
                 json.push_str(r#""inputs":"#);
                 let inputs_str = unsafe { cap.name("inputs").unwrap_unchecked() }.as_str();
-                for input_cap in Regex::new(r"\{[^\{\}]*\}").unwrap().find_iter(inputs_str) {
-                    if serde_json::from_str::<AbiIO>(input_cap.as_str()).is_err() {
-                        return Err("inputs are not valid format");
+                for cap in Regex::new(r"\{[^\{\}]*\}").unwrap().find_iter(inputs_str) {
+                    if let Ok(input_cap) = cap {
+                        if serde_json::from_str::<AbiIO>(input_cap.as_str()).is_err() {
+                            return Err("inputs are not valid format");
+                        }
                     }
                 }
                 json.push_str(inputs_str);
@@ -299,7 +466,7 @@ fn parse_fn_attr(fn_name: String, attr: String) -> Result<(Option<String>, Strin
                 json.push_str(r#""inputs":[],"#);
             }
 
-            if let Some(cap) = unsafe { Regex::new(r"name=(?P<name>[^,]*)").unwrap_unchecked() }
+            if let Ok(Some(cap)) = unsafe { Regex::new(r"name=(?P<name>[^,]*)").unwrap_unchecked() }
                 .captures(&attr_str)
             {
                 json.push_str(r#""name":""#);
@@ -309,15 +476,17 @@ fn parse_fn_attr(fn_name: String, attr: String) -> Result<(Option<String>, Strin
                 json.push_str(&format!(r#""name":"{}","#, fn_name.to_case(Camel)));
             }
 
-            if let Some(cap) =
-                unsafe { Regex::new(r"outputs=(?P<outputs>\[[^\[\]]*\])").unwrap_unchecked() }
+            if let Ok(Some(cap)) =
+                unsafe { Regex::new(r#"outputs=(?<outputs>\[.*?\])(?!")"#).unwrap_unchecked() }
                     .captures(&attr_str)
             {
                 json.push_str(r#""outputs":"#);
                 let outputs_str = unsafe { cap.name("outputs").unwrap_unchecked() }.as_str();
-                for output_cap in Regex::new(r"\{[^\{\}]*\}").unwrap().find_iter(outputs_str) {
-                    if serde_json::from_str::<AbiIO>(output_cap.as_str()).is_err() {
-                        return Err("outputs are not valid format");
+                for cap in Regex::new(r"\{[^\{\}]*\}").unwrap().find_iter(outputs_str) {
+                    if let Ok(output_cap) = cap {
+                        if serde_json::from_str::<AbiIO>(output_cap.as_str()).is_err() {
+                            return Err("outputs are not valid format");
+                        }
                     }
                 }
                 json.push_str(outputs_str);
@@ -326,7 +495,7 @@ fn parse_fn_attr(fn_name: String, attr: String) -> Result<(Option<String>, Strin
                 json.push_str(r#""outputs":[],"#);
             }
 
-            if let Some(cap) =
+            if let Ok(Some(cap)) =
                 unsafe { Regex::new(r"payable=(?P<payable>[^,]*)").unwrap_unchecked() }
                     .captures(&attr_str)
             {
@@ -339,7 +508,7 @@ fn parse_fn_attr(fn_name: String, attr: String) -> Result<(Option<String>, Strin
                 json.push_str(r#""payable":false,"#);
             }
 
-            if let Some(cap) = unsafe {
+            if let Ok(Some(cap)) = unsafe {
                 Regex::new(r"stateMutability=(?P<stateMutability>[^,]*)").unwrap_unchecked()
             }
             .captures(&attr_str)
@@ -354,10 +523,26 @@ fn parse_fn_attr(fn_name: String, attr: String) -> Result<(Option<String>, Strin
             }
 
             json.push_str(r#""type":"function"}"#);
-            Ok((Some(head.replace("\"", "")), json))
+            if head.starts_with("only_by") {
+                Ok((None, json, root_str))
+            } else {
+                Ok((Some(head.replace("\"", "")), json, root_str))
+            }
         }
+    } else if let Ok(Some(cap)) =
+        unsafe { Regex::new(r"only_by=(?P<account>[^,]*)").unwrap_unchecked() }.captures(&attr_str)
+    {
+        Ok((
+            None,
+            "{}".into(),
+            Some(
+                unsafe { cap.name("account").unwrap_unchecked() }
+                    .as_str()
+                    .into(),
+            ),
+        ))
     } else {
-        Ok((Some(attr_str.replace("\"", "")), "{}".into()))
+        Ok((Some(attr_str.replace("\"", "")), "{}".into(), None))
     };
 }
 
@@ -388,11 +573,12 @@ fn parse_fn_attr(fn_name: String, attr: String) -> Result<(Option<String>, Strin
 /// The functional signature can be specific as following ways.
 /// `#[ewasm_fn(00fdd58e)]` or #[ewasm_fn("00fdd58e")]
 ///
-/// ``` #[ewasm_fn(00fdd58e,
+/// ```compile_fail
+/// #[ewasm_fn(00fdd58e,
 /// constant=true,
 /// inputs=[
 ///     { "internalType": "address", "name": "account", "type": "address" },
-///     { "internalType": "uinit256", "name": "token_id", "type": "uinit256" }
+///     { "internalType": "uint256", "name": "token_id", "type": "uint256" }
 /// ],
 /// name=balanceOf,
 /// outputs=[
@@ -407,19 +593,33 @@ fn parse_fn_attr(fn_name: String, attr: String) -> Result<(Option<String>, Strin
 /// `outputs` are `[]`; the default value of `stateMutability` is `view`; the default name is the
 /// camel case style of the function name.
 ///
+/// The handler also can be restricted by called special account with `only_by` attribute,
+/// following are examples:
+/// ```compile_fail
+/// #[ewasm_fn(only_by=8663DBF0cC68AaF37fC8BA262F2df4c666a41993)]
+/// #[ewasm_fn(only_by="0x8663DBF0cC68AaF37fC8BA262F2df4c666a41993")]
+/// #[ewasm_fn(only_by=0x8663DBF0cC68AaF37fC8BA262F2df4c666a41993)]
+/// #[ewasm_fn(only_by="8663DBF0cC68AaF37fC8BA262F2df4c666a41993")]
+/// ```
 #[proc_macro_error]
 #[proc_macro_attribute]
 pub fn ewasm_fn(attr: TokenStream, item: TokenStream) -> TokenStream {
-    let input = syn::parse_macro_input!(item as syn::ItemFn);
-    let name = &input.sig.ident;
+    let syn::ItemFn {
+        attrs,
+        vis,
+        sig,
+        block,
+    } = syn::parse_macro_input!(item as syn::ItemFn);
+    let syn::Block { stmts, .. } = *block;
 
-    let (hex_str, abi_str) = match parse_fn_attr(name.to_string(), attr.to_string()) {
+    let name = &sig.ident;
+
+    let (hex_str, abi_str, root_str) = match parse_fn_attr(name.to_string(), attr.to_string()) {
         Ok(o) => o,
         Err(e) => abort_call_site!(e),
     };
 
-    let args = &input
-        .sig
+    let args = &sig
         .inputs
         .iter()
         .map(|fn_arg| match fn_arg {
@@ -477,13 +677,34 @@ pub fn ewasm_fn(attr: TokenStream, item: TokenStream) -> TokenStream {
         &format!("{}_SIG", name.to_string().to_ascii_uppercase()),
         Span::call_site(),
     );
-    let result = quote! {
-        pub const #sig_name : [u8; 4] = [#sig_0, #sig_1, #sig_2, #sig_3];
-        pub(crate) const #abi_info: &'static str = #abi_str;
+    let result = if let Some(root_str) = root_str {
+        let addr = root_str.replace("\"", "");
+        quote! {
+            pub const #sig_name : [u8; 4] = [#sig_0, #sig_1, #sig_2, #sig_3];
+            pub(crate) const #abi_info: &'static str = #abi_str;
 
-        #[cfg(target_arch = "wasm32")]
-        #[cfg(not(any(feature = "constructor", feature = "constructor-test")))]
-        #input
+            #[cfg(target_arch = "wasm32")]
+            #[cfg(not(any(feature = "constructor", feature = "constructor-test")))]
+            #(#attrs)*
+            #vis #sig {
+                if sewup::utils::caller() != sewup::types::Address::from_str(#addr)? {
+                    return Err(sewup::errors::HandlerError::Unauthorized.into())
+                }
+                #(#stmts)*
+            }
+        }
+    } else {
+        quote! {
+            pub const #sig_name : [u8; 4] = [#sig_0, #sig_1, #sig_2, #sig_3];
+            pub(crate) const #abi_info: &'static str = #abi_str;
+
+            #[cfg(target_arch = "wasm32")]
+            #[cfg(not(any(feature = "constructor", feature = "constructor-test")))]
+            #(#attrs)*
+            #vis #sig {
+                #(#stmts)*
+            }
+        }
     };
     result.into()
 }
@@ -554,7 +775,7 @@ pub fn ewasm_constructor(_attr: TokenStream, item: TokenStream) -> TokenStream {
 /// constant=true,
 /// inputs=[
 ///     { "internalType": "address", "name": "account", "type": "address" },
-///     { "internalType": "uinit256", "name": "token_id", "type": "uinit256" }
+///     { "internalType": "uint256", "name": "token_id", "type": "uint256" }
 /// ],
 /// name=balanceOf,
 /// outputs=[
@@ -575,7 +796,7 @@ pub fn ewasm_lib_fn(attr: TokenStream, item: TokenStream) -> TokenStream {
     let input = syn::parse_macro_input!(item as syn::ItemFn);
     let name = &input.sig.ident;
 
-    let (hex_str, abi_str) = match parse_fn_attr(name.to_string(), attr.to_string()) {
+    let (hex_str, abi_str, _root_str) = match parse_fn_attr(name.to_string(), attr.to_string()) {
         Ok(o) => o,
         Err(e) => abort_call_site!(e),
     };
@@ -720,7 +941,7 @@ pub fn ewasm_fn_sig(item: TokenStream) -> TokenStream {
 #[proc_macro]
 pub fn ewasm_input(item: TokenStream) -> TokenStream {
     let re = unsafe { Regex::new(r"(?P<instance>.*)\s+for\s+(?P<sig>.*)").unwrap_unchecked() };
-    if let Some(cap) = re.captures(&item.to_string()) {
+    if let Ok(Some(cap)) = re.captures(&item.to_string()) {
         let sig = unsafe { cap.name("sig").unwrap_unchecked() }.as_str();
         let instance = unsafe { cap.name("instance").unwrap_unchecked() }.as_str();
         let output = if instance == "None" {
@@ -779,7 +1000,7 @@ pub fn ewasm_input_from(item: TokenStream) -> TokenStream {
         Regex::new(r"^(?P<contract>\w+)\s+move\s+(?P<name>[^,]+),?(?P<error_handler>.*)")
             .unwrap_unchecked()
     };
-    if let Some(cap) = re.captures(&item.to_string()) {
+    if let Ok(Some(cap)) = re.captures(&item.to_string()) {
         let contract = Ident::new(
             unsafe { cap.name("contract").unwrap_unchecked() }.as_str(),
             Span::call_site(),
@@ -1371,7 +1592,9 @@ pub fn ewasm_test(attr: TokenStream, item: TokenStream) -> TokenStream {
         Regex::new(r"^fn (?P<fn_name>[^\(\s]*)(?P<to_first_bracket>[^\{]*\{)").unwrap_unchecked()
     };
     let context = item.to_string();
-    if mod_re.captures(&context).is_some() {
+    let mod_captures = mod_re.captures(&context).unwrap();
+    let fn_captures = fn_re.captures(&context).unwrap();
+    if mod_captures.is_some() {
         let attr_str = attr.to_string().replace(" ", "");
         let runtime_log_option = if attr_str.is_empty() {
             "".to_string()
@@ -1392,6 +1615,7 @@ pub fn ewasm_test(attr: TokenStream, item: TokenStream) -> TokenStream {
                 use std::path::PathBuf;
                 use std::process::Command;
                 use std::sync::Arc;
+                use sewup::runtimes::traits::RT;
 
                 fn _build_wasm(opt: Option<String>) -> String {
                     let cargo_cmd = format!("cargo build --release --target=wasm32-unknown-unknown {}", opt.unwrap_or_default());
@@ -1489,7 +1713,7 @@ pub fn ewasm_test(attr: TokenStream, item: TokenStream) -> TokenStream {
             .to_string()
             .parse()
             .unwrap();
-    } else if fn_re.captures(&context).is_some() {
+    } else if fn_captures.is_some() {
         let attr_str = attr.to_string().replace(" ", "");
         if !attr_str.is_empty() {
             abort_call_site!("no support option when wrapping on function")
@@ -1534,7 +1758,7 @@ pub fn ewasm_assert_eq(item: TokenStream) -> TokenStream {
     let re = unsafe {
         Regex::new(r#"^(?P<fn_name>[^(]+?)\((?P<params>[^)]*?)\)\s*(by)?\s*(?P<caller>"[^"]*")?\s*,(?P<equivalence>.*)"#).unwrap_unchecked()
     };
-    if let Some(cap) = re.captures(&item.to_string().replace("\n", "")) {
+    if let Ok(Some(cap)) = re.captures(&item.to_string().replace("\n", "")) {
         let fn_name = unsafe { cap.name("fn_name").unwrap_unchecked() }
             .as_str()
             .replace(" ", "");
@@ -1578,7 +1802,7 @@ pub fn ewasm_auto_assert_eq(item: TokenStream) -> TokenStream {
     let re = unsafe {
         Regex::new(r#"^(?P<fn_name>[^(]+?)\((?P<params>[^)]*?)\)\s*(by)?\s*(?P<caller>"[^"]*")?\s*,(?P<equivalence>.*)"#).unwrap_unchecked()
     };
-    if let Some(cap) = re.captures(&item.to_string().replace("\n", "")) {
+    if let Ok(Some(cap)) = re.captures(&item.to_string().replace("\n", "")) {
         let fn_name = unsafe { cap.name("fn_name").unwrap_unchecked() }.as_str();
         let params = unsafe { cap.name("params").unwrap_unchecked() }
             .as_str()
@@ -1636,7 +1860,7 @@ pub fn ewasm_assert_ok(item: TokenStream) -> TokenStream {
         )
         .unwrap_unchecked()
     };
-    if let Some(cap) = re.captures(&item.to_string().replace("\n", "")) {
+    if let Ok(Some(cap)) = re.captures(&item.to_string().replace("\n", "")) {
         let fn_name = unsafe { cap.name("fn_name").unwrap_unchecked() }.as_str();
         let params = unsafe { cap.name("params").unwrap_unchecked() }
             .as_str()
@@ -1680,7 +1904,7 @@ pub fn ewasm_rusty_assert_ok(item: TokenStream) -> TokenStream {
         )
         .unwrap_unchecked()
     };
-    if let Some(cap) = re.captures(&item.to_string().replace("\n", "")) {
+    if let Ok(Some(cap)) = re.captures(&item.to_string().replace("\n", "")) {
         let fn_name = unsafe { cap.name("fn_name").unwrap_unchecked() }.as_str();
         let params = unsafe { cap.name("params").unwrap_unchecked() }
             .as_str()
@@ -1806,3 +2030,22 @@ pub fn ewasm_call_only_by(item: TokenStream) -> TokenStream {
 
     output.into()
 }
+
+/// helps you inspect the storage in test
+/// this will print out the detail of storage, you can add description on the storage debug or
+/// leave  it empty
+/// ```compile_fail
+/// ewasm_storage_debug!();
+/// ewasm_storage_debug!(description);
+/// ```
+#[proc_macro]
+pub fn ewasm_storage_debug(item: TokenStream) -> TokenStream {
+    let desc = item.to_string();
+    quote!({
+        let __cloned = _runtime.clone();
+        let __borrowed = __cloned.borrow();
+        let __storage = __borrowed.get_storage(&[0; 20]);
+        eprintln!("{}", sewup::utils::pretty_print_storage(#desc, __storage));
+    })
+    .into()
+}
